@@ -7,12 +7,12 @@
  * Copyright (c) 1987-1994 The Regents of the University of California.
  * Copyright (c) 1993-1996 Lucent Technologies.
  * Copyright (c) 1994-1998 Sun Microsystems, Inc.
- * Copyright (c) 1998-1999 by Scriptics Corporation.
+ * Copyright (c) 1998-2000 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tcl.h,v 1.57.2.3 1999/11/19 23:32:02 hobbs Exp $
+ * RCS: @(#) $Id: tcl.h,v 1.70 2000/04/19 08:32:44 hobbs Exp $
  */
 
 #ifndef _TCL
@@ -41,25 +41,28 @@ extern "C" {
  * library/init.tcl	(only if Major.minor changes, not patchlevel) 1 LOC
  * unix/configure.in	(2 LOC Major, 2 LOC minor, 1 LOC patch)
  * win/configure.in	(as above)
+ * win/tcl.m4		(not patchlevel)
  * win/makefile.vc	(not patchlevel) 2 LOC
  * win/pkgIndex.tcl	(not patchlevel, for tclregNN.dll)
  * README		(sections 0 and 2)
- * mac/README		(2 LOC)
+ * mac/README		(2 LOC, not patchlevel)
  * win/README.binary	(sections 0-4)
  * win/README		(not patchlevel) (sections 0 and 2)
  * unix/README		(not patchlevel) (part (h))
+ * unix/tcl.spec	(2 LOC Major/Minor, 1 LOC patch)
  * tests/basic.test	(not patchlevel) (version checks)
  * tools/tcl.hpj.in	(not patchlevel, for windows installer)
  * tools/tcl.wse.in	(for windows installer)
+ * tools/tclSplash.bmp	(not patchlevel)
  */
 
 #define TCL_MAJOR_VERSION   8
-#define TCL_MINOR_VERSION   2
+#define TCL_MINOR_VERSION   3
 #define TCL_RELEASE_LEVEL   TCL_FINAL_RELEASE
-#define TCL_RELEASE_SERIAL  3
+#define TCL_RELEASE_SERIAL  1
 
-#define TCL_VERSION	    "8.2"
-#define TCL_PATCH_LEVEL	    "8.2.3+"
+#define TCL_VERSION	    "8.3"
+#define TCL_PATCH_LEVEL	    "8.3.1+"
 
 /*
  * The following definitions set up the proper options for Windows
@@ -161,8 +164,10 @@ extern "C" {
 #ifndef TCL_THREADS
 #define Tcl_MutexLock(mutexPtr)
 #define Tcl_MutexUnlock(mutexPtr)
+#define Tcl_MutexFinalize(mutexPtr)
 #define Tcl_ConditionNotify(condPtr)
 #define Tcl_ConditionWait(condPtr, mutexPtr, timePtr)
+#define Tcl_ConditionFinalize(condPtr)
 #endif /* TCL_THREADS */
 
 /* 
@@ -280,6 +285,13 @@ extern "C" {
 #   define CONST
 #endif
 
+/*
+ * Make sure EXTERN isn't defined elsewhere
+ */
+#ifdef EXTERN
+#undef EXTERN
+#endif /* EXTERN */
+
 #ifdef __cplusplus
 #   define EXTERN extern "C" TCL_STORAGE_CLASS
 #else
@@ -379,6 +391,49 @@ typedef struct Tcl_TimerToken_ *Tcl_TimerToken;
 typedef struct Tcl_Trace_ *Tcl_Trace;
 typedef struct Tcl_Var_ *Tcl_Var;
 
+/*
+ * Definition of the interface to procedures implementing threads.
+ * A procedure following this definition is given to each call of
+ * 'Tcl_CreateThread' and will be called as the main fuction of
+ * the new thread created by that call.
+ */
+
+#ifdef MAC_TCL
+typedef pascal void *(Tcl_ThreadCreateProc) _ANSI_ARGS_((ClientData clientData));
+#elif defined __WIN32__
+typedef unsigned (__stdcall Tcl_ThreadCreateProc) _ANSI_ARGS_((ClientData clientData));
+#else
+typedef void (Tcl_ThreadCreateProc) _ANSI_ARGS_((ClientData clientData));
+#endif
+
+
+/*
+ * Threading function return types used for abstracting away platform
+ * differences when writing a Tcl_ThreadCreateProc.  See the NewThread
+ * function in generic/tclThreadTest.c for it's usage.
+ */
+#ifdef MAC_TCL
+#   define Tcl_ThreadCreateType		pascal void *
+#   define TCL_THREAD_CREATE_RETURN	return NULL
+#elif defined __WIN32__
+#   define Tcl_ThreadCreateType		unsigned __stdcall
+#   define TCL_THREAD_CREATE_RETURN	return 0
+#else
+#   define Tcl_ThreadCreateType		void
+#   define TCL_THREAD_CREATE_RETURN	
+#endif
+
+
+
+/*
+ * Definition of values for default stacksize and the possible flags to be
+ * given to Tcl_CreateThread.
+ */
+
+#define TCL_THREAD_STACK_DEFAULT (0)    /* Use default size for stack */
+#define TCL_THREAD_NOFLAGS       (0000) /* Standard flags, default behaviour */
+#define TCL_THREAD_JOINABLE      (0001) /* Mark the thread as joinable */
+
 /*
  * Flag values passed to Tcl_GetRegExpFromObj.
  */
@@ -723,7 +778,7 @@ typedef struct Tcl_SavedResult {
 /*
  * The following definitions support Tcl's namespace facility.
  * Note: the first five fields must match exactly the fields in a
- * Namespace structure (see tcl.h). 
+ * Namespace structure (see tclInt.h). 
  */
 
 typedef struct Tcl_Namespace {
