@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclCmdAH.c,v 1.7.4.1 1999/09/22 04:12:45 hobbs Exp $
+ * RCS: @(#) $Id: tclCmdAH.c,v 1.12 2000/01/21 02:25:26 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -257,7 +257,7 @@ Tcl_CatchObjCmd(dummy, interp, objc, objv)
     if (objc == 3) {
 	varNamePtr = objv[2];
     }
-    
+
     result = Tcl_EvalObjEx(interp, objv[1], 0);
     
     if (objc == 3) {
@@ -613,17 +613,15 @@ Tcl_EvalObjCmd(dummy, interp, objc, objv)
     }
     
     if (objc == 2) {
-	result = Tcl_EvalObjEx(interp, objv[1], 0);
+	result = Tcl_EvalObjEx(interp, objv[1], TCL_EVAL_DIRECT);
     } else {
 	/*
 	 * More than one argument: concatenate them together with spaces
-	 * between, then evaluate the result.
+	 * between, then evaluate the result.  Tcl_EvalObjEx will delete
+	 * the object when it decrements its refcount after eval'ing it.
 	 */
-    
-	objPtr = Tcl_ConcatObj(objc-1, objv+1);
-	Tcl_IncrRefCount(objPtr);
-	result = Tcl_EvalObjEx(interp, objPtr, 0);
-	Tcl_DecrRefCount(objPtr);
+    	objPtr = Tcl_ConcatObj(objc-1, objv+1);
+	result = Tcl_EvalObjEx(interp, objPtr, TCL_EVAL_DIRECT);
     }
     if (result == TCL_ERROR) {
 	char msg[32 + TCL_INTEGER_SPACE];
@@ -793,7 +791,8 @@ Tcl_FileObjCmd(dummy, interp, objc, objv)
  */
 
     static char *fileOptions[] = {
-	"atime",	"attributes",	"copy",		"delete",
+	"atime",	"attributes",	"channels",	"copy",
+	"delete",
 	"dirname",	"executable",	"exists",	"extension",
 	"isdirectory",	"isfile",	"join",		"lstat",
 	"mtime",	"mkdir",	"nativename",	"owned",
@@ -803,7 +802,8 @@ Tcl_FileObjCmd(dummy, interp, objc, objv)
 	(char *) NULL
     };
     enum options {
-	FILE_ATIME,	FILE_ATTRIBUTES, FILE_COPY,	FILE_DELETE,
+	FILE_ATIME,	FILE_ATTRIBUTES, FILE_CHANNELS,	FILE_COPY,
+	FILE_DELETE,
 	FILE_DIRNAME,	FILE_EXECUTABLE, FILE_EXISTS,	FILE_EXTENSION,
 	FILE_ISDIRECTORY, FILE_ISFILE,	FILE_JOIN,	FILE_LSTAT,
 	FILE_MTIME,	FILE_MKDIR,	FILE_NATIVENAME, FILE_OWNED,
@@ -825,19 +825,55 @@ Tcl_FileObjCmd(dummy, interp, objc, objv)
     switch ((enum options) index) {
     	case FILE_ATIME: {
 	    struct stat buf;
-	    
-	    if (objc != 3) {
-		goto only3Args;
+	    char *fileName;
+	    struct utimbuf tval;
+
+	    if ((objc < 3) || (objc > 4)) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name ?time?");
+		return TCL_ERROR;
 	    }
 	    if (GetStatBuf(interp, objv[2], TclStat, &buf) != TCL_OK) {
 		return TCL_ERROR;
 	    }
+	    if (objc == 4) {
+		if (Tcl_GetLongFromObj(interp, objv[3],
+			(long*)(&buf.st_atime)) != TCL_OK) {
+		    return TCL_ERROR;
+		}
+		tval.actime = buf.st_atime;
+		tval.modtime = buf.st_mtime;
+		fileName = Tcl_GetString(objv[2]);
+		if (utime(fileName, &tval) != 0) {
+		    Tcl_AppendStringsToObj(resultPtr,
+			    "could not set access time for file \"",
+			    fileName, "\": ",
+			    Tcl_PosixError(interp), (char *) NULL);
+		    return TCL_ERROR;
+		}
+		/*
+		 * Do another stat to ensure that the we return the
+		 * new recognized atime - hopefully the same as the
+		 * one we sent in.  However, fs's like FAT don't
+		 * even know what atime is.
+		 */
+		if (GetStatBuf(interp, objv[2], TclStat, &buf) != TCL_OK) {
+		    return TCL_ERROR;
+		}
+	    }
 	    Tcl_SetLongObj(resultPtr, (long) buf.st_atime);
 	    return TCL_OK;
 	}
 	case FILE_ATTRIBUTES: {
             return TclFileAttrsCmd(interp, objc, objv);
 	}
+	case FILE_CHANNELS: {
+	    if ((objc < 2) || (objc > 3)) {
+		Tcl_WrongNumArgs(interp, 2, objv, "?pattern?");
+		return TCL_ERROR;
+	    }
+	    return Tcl_GetChannelNamesEx(interp,
+		    ((objc == 2) ? NULL : Tcl_GetString(objv[2])));
+	}
 	case FILE_COPY: {
 	    int result;
 	    char **argv;
@@ -977,13 +1013,41 @@ Tcl_FileObjCmd(dummy, interp, objc, objv)
 	}
 	case FILE_MTIME: {
 	    struct stat buf;
-	    
-	    if (objc != 3) {
-		goto only3Args;
+	    char *fileName;
+	    struct utimbuf tval;
+
+	    if ((objc < 3) || (objc > 4)) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name ?time?");
+		return TCL_ERROR;
 	    }
 	    if (GetStatBuf(interp, objv[2], TclStat, &buf) != TCL_OK) {
 		return TCL_ERROR;
 	    }
+	    if (objc == 4) {
+		if (Tcl_GetLongFromObj(interp, objv[3],
+			(long*)(&buf.st_mtime)) != TCL_OK) {
+		    return TCL_ERROR;
+		}
+		tval.actime = buf.st_atime;
+		tval.modtime = buf.st_mtime;
+		fileName = Tcl_GetString(objv[2]);
+		if (utime(fileName, &tval) != 0) {
+		    Tcl_AppendStringsToObj(resultPtr,
+			    "could not set modification time for file \"",
+			    fileName, "\": ",
+			    Tcl_PosixError(interp), (char *) NULL);
+		    return TCL_ERROR;
+		}
+		/*
+		 * Do another stat to ensure that the we return the
+		 * new recognized atime - hopefully the same as the
+		 * one we sent in.  However, fs's like FAT don't
+		 * even know what atime is.
+		 */
+		if (GetStatBuf(interp, objv[2], TclStat, &buf) != TCL_OK) {
+		    return TCL_ERROR;
+		}
+	    }
 	    Tcl_SetLongObj(resultPtr, (long) buf.st_mtime);
 	    return TCL_OK;
 	}
@@ -1910,7 +1974,7 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
     int gotPrecision;		/* Non-zero indicates that a precision has
 				 * been set for the current field. */
     int gotZero;		/* Non-zero indicates that a zero flag has
- 				 * been seen in the current field. */
+				 * been seen in the current field. */
 
     /*
      * This procedure is a bit nasty.  The goal is to use sprintf to
@@ -2008,13 +2072,13 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 	    if (*format == '-') {
 		gotMinus = 1;
 	    }
- 	    if (*format == '0') {
- 		/*
- 		 * This will be handled by sprintf for numbers, but we
- 		 * need to do the char/string ones ourselves
- 		 */
- 		gotZero = 1;
- 	    }
+	    if (*format == '0') {
+		/*
+		 * This will be handled by sprintf for numbers, but we
+		 * need to do the char/string ones ourselves
+		 */
+		gotZero = 1;
+	    }
 	    *newPtr = *format;
 	    newPtr++;
 	    format++;
