@@ -55,8 +55,8 @@ char *dlerror() {
     return "dynamic loading is not currently available on this system";
 }
   
-! int dlclose(handle)
-!     VOID *handle;
-  {
-!     return -1;
-  }
+int dlclose(handle)
+    VOID *handle;
+{
+    return -1;
+}
