@@ -369,7 +369,6 @@ TclpSetInitialEncodings()
     int i;
     Tcl_Obj *pathPtr;
     char *langEnv;
-    Tcl_DString ds;
 
     /*
      * Determine the current encoding from the LC_* or LANG environment
@@ -411,6 +410,7 @@ TclpSetInitialEncodings()
 		}
 	    }
 	    if (*p != '\0') {
+		Tcl_DString ds;
 		Tcl_DStringInit(&ds);
 		Tcl_DStringAppend(&ds, p, -1);
 
@@ -431,21 +431,64 @@ TclpSetInitialEncodings()
 
     Tcl_SetSystemEncoding(NULL, encoding);
 
+#if 0
     /*
-     * Hey, Belows are helpful for X11 XIM, NOT!!!!
+     * Initialize the C library's locale subsystem.  This is required
+     * for input methods to work properly on X11. Note that we need to
+     * retore the initial "C" locale so that Tcl can parse numbers
+     * properly.  The side effect of setting the default locale should be to
+     * load any locale specific modules that are needed by X.
+     */
+ 
+    Tcl_DStringInit(&ds);
+    Tcl_DStringAppend(&ds, setlocale(LC_ALL, NULL), -1);
+    setlocale(LC_ALL, "");
+    setlocale(LC_ALL, Tcl_DStringValue(&ds));
+    Tcl_DStringFree(&ds);
+#else
+    /*
+     * Hey, aboves are helpful for X11 XIM, NOT!!!!
+     *
      * If setlocale(LC_ALL, "") are once called, NOT reset to "C"
      * BECAUSE OF X11 uses locale sensitive libc routines such as:
+     * 
      *		mblen()
      *		mbstowcs()
      *		wcstombs()
+     *
      * Calling setlocale() with proper locale is of course needed,
      * futhermore, keeping the proper locale setting is ALSO needed
      * for proper XIM operation. If not, in the worst case, Tk dumps
      * core.
      *
-     * So Scott, Don't call setlocale() here.
-     *		m-hirano
+     * And here's another story:
+     *
+     * X11[56]'s internationalization depends on "locale model"
+     * supported by C language (runtime). But unfortunately, on some
+     * (many) systems, setlocale(3) supports only "C" locale. For such
+     * a system, X11 library has _Xsetlocale() (and some locale
+     * sensitive C runtime function like the mblen(), even If these
+     * function are not well-implemented, sigh). If on such a system,
+     * we should't call:
+     *
+     *		setlocale(LC_ALL)
+     *			or
+     *		setlocale(LC_CTYPE)
+     *
+     * at here because Tcl won't be linked with X11 libraries and
+     * nobody want to use the Tcl library that needs X11 libraries at
+     * execution/linkage time. Instead, call setlocale() (expanded to
+     * _Xsetlocale() with C preprocessor macro in <X11/Xlocale.h> if
+     * X_LOCLAE C preprocessor macro is defined, normaly done in Imake
+     * environment) at the initializing of Tk. So, at here, I just
+     * call only setlocale(LC_NUMERIC) since X11's internationalize
+     * implementation only depends on LC_CTYPE categoly (of cource
+     * LC_ALL includes this), This must be harmless and might let the
+     * Tcl and the Tk with XIM be both happy :)
+     *
+     *	m-hirano
      */
+#endif /* 0 */
 
     /*
      * In case the initial locale is not "C", ensure that the numeric
