@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclXtTest.c,v 1.3 1999/04/16 00:48:06 stanton Exp $
+ * RCS: @(#) $Id: tclXtTest.c,v 1.4 1999/07/02 06:05:34 welch Exp $
  */
 
 #include <X11/Intrinsic.h>
@@ -16,13 +16,15 @@
 
 static int	TesteventloopCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern void	InitNotifier _ANSI_ARGS_((void));
+EXTERN void	InitNotifier _ANSI_ARGS_((void));
+EXTERN int	Xttest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int	Tclxttest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 
 
 /*
  *----------------------------------------------------------------------
  *
- * Xttest_Init --
+ * Xttest_Init, Tclxttest_Init --
  *
  *	This procedure performs application-specific initialization.
  *	Most applications, especially those that incorporate additional
@@ -37,6 +39,12 @@ extern void	InitNotifier _ANSI_ARGS_((void));
  *
  *----------------------------------------------------------------------
  */
+int
+Tclxttest_Init(interp)
+    Tcl_Interp *interp;		/* Interpreter for application. */
+{
+    return Xttest_Init(interp);
+}
 
 int
 Xttest_Init(interp)
@@ -49,8 +57,9 @@ Xttest_Init(interp)
     InitNotifier();
     Tcl_CreateCommand(interp, "testeventloop", TesteventloopCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
-    return TCL_OK;
+    return Tcl_PkgProvide(interp, "xttest", "1.0");
 }
+
 
 /*
  *----------------------------------------------------------------------
