@@ -135,14 +135,6 @@ typedef struct ConsoleEvent {
  * Declarations for functions used only in this file.
  */
 
-static int		ApplicationType(Tcl_Interp *interp,
-			    const char *fileName, char *fullName);
-static void		BuildCommandLine(const char *executable, int argc, 
-			    char **argv, Tcl_DString *linePtr);
-static void		CopyChannel(HANDLE dst, HANDLE src);
-static BOOL		HasConsole(void);
-static TclFile		MakeFile(HANDLE handle);
-static char *		MakeTempFile(Tcl_DString *namePtr);
 static int		ConsoleBlockModeProc(ClientData instanceData, int mode);
 static void		ConsoleCheckProc(ClientData clientData, int flags);
 static int		ConsoleCloseProc(ClientData instanceData,
@@ -161,7 +153,6 @@ static void		ConsoleSetupProc(ClientData clientData, int flags);
 static void		ConsoleWatchProc(ClientData instanceData, int mask);
 static DWORD WINAPI	ConsoleWriterThread(LPVOID arg);
 static void		ProcExitHandler(ClientData clientData);
-static int		TempFileName(WCHAR name[MAX_PATH]);
 static int		WaitForRead(ConsoleInfo *infoPtr, int blocking);
 
 /*
@@ -632,11 +623,11 @@ ConsoleInputProc(
 	 */
 
 	if ((unsigned int) bufSize < (infoPtr->bytesRead - infoPtr->offset)) {
-	    memcpy(buf, &infoPtr->buffer[infoPtr->offset], bufSize);
+	    memcpy(buf, &infoPtr->buffer[infoPtr->offset], (unsigned int) bufSize);
 	    bytesRead = bufSize;
 	    infoPtr->offset += bufSize;
 	} else {
-	    memcpy(buf, &infoPtr->buffer[infoPtr->offset], bufSize);
+	    memcpy(buf, &infoPtr->buffer[infoPtr->offset], (unsigned int) bufSize);
 	    bytesRead = infoPtr->bytesRead - infoPtr->offset;
 
 	    /*
@@ -730,9 +721,9 @@ ConsoleOutputProc(
 		ckfree(infoPtr->writeBuf);
 	    }
 	    infoPtr->writeBufLen = toWrite;
-	    infoPtr->writeBuf = ckalloc(toWrite);
+	    infoPtr->writeBuf = ckalloc((unsigned int) toWrite);
 	}
-	memcpy(infoPtr->writeBuf, buf, toWrite);
+	memcpy(infoPtr->writeBuf, buf, (unsigned int) toWrite);
 	infoPtr->toWrite = toWrite;
 	ResetEvent(infoPtr->writable);
 	SetEvent(infoPtr->startWriter);
