@@ -4,6 +4,8 @@
 
 #include <tclWinInt.h>
 
+#ifndef STATIC_BUILD
+
 #if defined(_MSC_VER) || defined(__GNUC__)
 #define DllEntryPoint DllMain
 #endif
@@ -47,3 +49,5 @@ DllEntryPoint(
 
     return TRUE; 
 }
+#endif /* !STATIC_BUILD */
+
