@@ -11,14 +11,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinFile.c,v 1.6 1999/04/21 21:50:34 rjohnson Exp $
+ * RCS: @(#) $Id: tclWinFile.c,v 1.7 1999/12/12 22:46:51 hobbs Exp $
  */
 
 #include "tclWinInt.h"
 #include <sys/stat.h>
-#ifndef __GNUC__
 #include <shlobj.h>
-#endif
 #include <lmaccess.h>		/* For TclpGetUserHome(). */
 
 static time_t		ToCTime(FILETIME fileTime);
@@ -129,14 +127,11 @@ TclpMatchFilesTypes(
     DWORD attr, volFlags;
     HANDLE handle;
     WIN32_FIND_DATAT data;
-    BOOL found;
+    BOOL found = 0;
     Tcl_DString ds;
     TCHAR *nativeName;
     Tcl_Obj *resultPtr;
 
-    /* Until we actually handle types, we do this to avoid warnings */
-    types = NULL;     
-
     /*
      * Convert the path to normalized form since some interfaces only
      * accept backslashes.  Also, ensure that the directory ends with a
@@ -282,7 +277,7 @@ TclpMatchFilesTypes(
     for (found = 1; found != 0; 
 	    found = (*tclWinProcs->findNextFileProc)(handle, &data)) {
 	TCHAR *nativeMatchResult;
-	char *name;
+	char *name, *fname;
 
 	if (tclWinProcs->useWide) {
 	    nativeName = (TCHAR *) data.w.cFileName;
@@ -329,21 +324,76 @@ TclpMatchFilesTypes(
 	Tcl_DStringAppend(dirPtr, name, -1);
 	Tcl_DStringFree(&ds);
 
-	if (tail == NULL) {
-            Tcl_ListObjAppendElement(interp, resultPtr, 
-				     Tcl_NewStringObj(Tcl_DStringValue(dirPtr),Tcl_DStringLength(dirPtr)));
-	} else {
-	    nativeName = Tcl_WinUtfToTChar(Tcl_DStringValue(dirPtr), 
-		    Tcl_DStringLength(dirPtr), &ds);
-	    attr = (*tclWinProcs->getFileAttributesProc)(nativeName);
-	    Tcl_DStringFree(&ds);
+	fname = Tcl_DStringValue(dirPtr);
+	nativeName = Tcl_WinUtfToTChar(fname, Tcl_DStringLength(dirPtr), &ds);
+	attr = (*tclWinProcs->getFileAttributesProc)(nativeName);
+	Tcl_DStringFree(&ds);
 
-	    if (attr & FILE_ATTRIBUTE_DIRECTORY) {
-		Tcl_DStringAppend(dirPtr, "/", 1);
-		result = TclDoGlob(interp, separators, dirPtr, tail, types);
-		if (result != TCL_OK) {
-		    break;
+	if (tail == NULL) {
+	    int typeOk = 1;
+	    if (types != NULL) {
+		if (types->perm != 0) {
+		    if (
+			((types->perm & TCL_GLOB_PERM_RONLY) &&
+				!(attr & FILE_ATTRIBUTE_READONLY)) ||
+			((types->perm & TCL_GLOB_PERM_HIDDEN) &&
+				!(attr & FILE_ATTRIBUTE_HIDDEN)) ||
+			((types->perm & TCL_GLOB_PERM_R) &&
+				(TclpAccess(fname, R_OK) != 0)) ||
+			((types->perm & TCL_GLOB_PERM_W) &&
+				(TclpAccess(fname, W_OK) != 0)) ||
+			((types->perm & TCL_GLOB_PERM_X) &&
+				(TclpAccess(fname, X_OK) != 0))
+			) {
+			typeOk = 0;
+		    }
 		}
+		if (typeOk && types->type != 0) {
+		    struct stat buf;
+		    /*
+		     * We must match at least one flag to be listed
+		     */
+		    typeOk = 0;
+		    if (TclpLstat(fname, &buf) >= 0) {
+			/*
+			 * In order bcdpfls as in 'find -t'
+			 */
+			if (
+			    ((types->type & TCL_GLOB_TYPE_BLOCK) &&
+				    S_ISBLK(buf.st_mode)) ||
+			    ((types->type & TCL_GLOB_TYPE_CHAR) &&
+				    S_ISCHR(buf.st_mode)) ||
+			    ((types->type & TCL_GLOB_TYPE_DIR) &&
+				    S_ISDIR(buf.st_mode)) ||
+			    ((types->type & TCL_GLOB_TYPE_PIPE) &&
+				    S_ISFIFO(buf.st_mode)) ||
+			    ((types->type & TCL_GLOB_TYPE_FILE) &&
+				    S_ISREG(buf.st_mode))
+#ifdef S_ISLNK
+			    || ((types->type & TCL_GLOB_TYPE_LINK) &&
+				    S_ISLNK(buf.st_mode))
+#endif
+#ifdef S_ISSOCK
+			    || ((types->type & TCL_GLOB_TYPE_SOCK) &&
+				    S_ISSOCK(buf.st_mode))
+#endif
+			    ) {
+			    typeOk = 1;
+			}
+		    } else {
+			/* Posix error occurred */
+		    }
+		}		
+	    } 
+	    if (typeOk) {
+		Tcl_ListObjAppendElement(interp, resultPtr, 
+			Tcl_NewStringObj(fname, Tcl_DStringLength(dirPtr)));
+	    }
+	} else if (attr & FILE_ATTRIBUTE_DIRECTORY) {
+	    Tcl_DStringAppend(dirPtr, "/", 1);
+	    result = TclDoGlob(interp, separators, dirPtr, tail, types);
+	    if (result != TCL_OK) {
+		break;
 	    }
 	}
 	Tcl_DStringSetLength(dirPtr, dirLength);
